@@ -20,6 +20,9 @@ test_that("existing non-variables get inlined", {
   n <- 10
   expect_equal(capture_dot(dt, x + n), quote(x + 10))
   expect_equal(capture_dot(dt, x + m), quote(x + m))
+  # even if they start with "." (#386)
+  .n <- 20
+  expect_equal(capture_dot(dt, x + .n), quote(x + 20))
 })
 
 test_that("unless we're operating in the global environment", {
@@ -30,6 +33,20 @@ test_that("unless we're operating in the global environment", {
   expect_equal(capture_dot(dt, !!quo, j = FALSE), quote(x + n))
 })
 
+test_that("ignores accessor calls, #434", {
+  df <- tibble(length = 1)
+
+  step <- lazy_dt(tibble(x = 1:3), "DT") %>%
+    mutate(y = df$length)
+
+  expect_equal(show_query(step), expr(copy(DT)[, `:=`(y = df$length)]))
+
+  step <- lazy_dt(tibble(x = 1:3), "DT") %>%
+    mutate(y = df[["length"]])
+
+  expect_equal(show_query(step), expr(copy(DT)[, `:=`(y = df[["length"]])]))
+})
+
 test_that("using environment of inlined quosures", {
   dt <- lazy_dt(data.frame(x = 1:10, y = 1:10))
 
@@ -70,14 +87,17 @@ test_that("translates if_else()/ifelse()", {
   )
 
   # Handles unusual argument names/order
-  expect_equal(
-    capture_dot(df, ifelse(x < 0, n = 2, yes = 1)),
-    expr(fifelse(x < 0, 1, 2))
-  )
-  expect_equal(
-    capture_dot(df, if_else(x < 0, f = 2, true = 1)),
-    expr(fifelse(x < 0, 1, 2))
-  )
+  suppressWarnings({
+    expect_equal(
+      capture_dot(df, ifelse(x < 0, n = 2, yes = 1)),
+      expr(fifelse(x < 0, 1, 2))
+    )
+    expect_equal(
+      capture_dot(df, if_else(x < 0, f = 2, true = 1)),
+      expr(fifelse(x < 0, 1, 2))
+    )
+  })
+
 
   # tidyeval works inside if_else, #220
   expect_equal(
@@ -121,17 +141,49 @@ test_that("translates case_when()", {
     quote(fcase(x1, y1, x2, y2, x3, TRUE, rep(TRUE, .N), y4))
   )
 
+  # can use `.default` and `.default` doesn't need to be in last position, #429
+  expect_equal(
+    capture_dot(dt, case_when(x1 ~ y1, x2 ~ y2, .default = y4, x3 ~ TRUE)),
+    quote(fcase(x1, y1, x2, y2, x3, TRUE, rep(TRUE, .N), y4))
+  )
+
   # translates recursively
   expect_equal(
     capture_dot(dt, case_when(x == 1 ~ n())),
     quote(fcase(x == 1, .N))
   )
+
+  # Errors on `.ptype`
+  expect_error(
+    capture_dot(dt, case_when(x1 ~ y1, .ptype = double()))
+  )
+
+  # Errors on `.size`
+  expect_error(
+    capture_dot(dt, case_when(x1 ~ y1, .size = 1))
+  )
+})
+
+test_that("translates case_match()", {
+  dt <- lazy_dt(data.frame(x = 1:5))
+
+  # Works without `.default`
+  expect_equal(
+    capture_dot(dt, case_match(x, c(1, 2) ~ 1, 3 ~ 2)),
+    quote(fcase(x %in% c(1, 2), 1, x == 3, 2))
+  )
+
+  # Works with `.default`
+  expect_equal(
+    capture_dot(dt, case_match(x, c(1, 2) ~ 1, 3 ~ 2, .default = 3)),
+    quote(fcase(x %in% c(1, 2), 1, x == 3, 2, rep(TRUE, .N), 3))
+  )
 })
 
 test_that("translates lag()/lead()", {
   df <- data.frame(x = 1:5, y = 1:5)
   expect_equal(
-    capture_dot(df, lag(x)),
+    capture_dot(df, lag(.data$x)),
     expr(shift(x, type = "lag"))
   )
   expect_equal(
@@ -176,6 +228,39 @@ test_that("can splice a data frame", {
   expect_equal(dots, as.list(df))
 })
 
+test_that("can use glue, (#344)", {
+  df <- data.table(a = letters[1:3], b = letters[1:3])
+  expect_equal(
+    capture_dot(df, glue::glue("{a}_{b}")),
+    quote(glue::glue("{a}_{b}", .envir = .SD))
+  )
+  expect_equal(
+    capture_dot(df, glue::glue("{a}_{b}"), j = FALSE),
+    quote(glue::glue("{a}_{b}"))
+  )
+
+  out <- df %>%
+    transmute(a_b = glue::glue("{a}_{b}")) %>%
+    collect()
+  expect_equal(out$a_b, c("a_a", "b_b", "c_c"))
+})
+
+test_that("properly handles anonymous functions, #362", {
+  df <- data.table(a = list(1, 1, 1))
+  expect_equal(
+    capture_dot(df, sapply(a, function(x) x + n())),
+    quote(sapply(a, function(x) x + .N))
+  )
+})
+
+test_that("translates `consecutive_id()`", {
+  df <- data.frame(x = 1:5, y = 1:5)
+  expect_equal(
+    capture_dot(df, consecutive_id(x, y)),
+    expr(rleid(x, y))
+  )
+})
+
 # evaluation --------------------------------------------------------------
 
 test_that("can access functions in local env", {
@@ -209,6 +294,16 @@ test_that("locals are executed before call", {
   )
 })
 
+test_that("errors when `where()` is used, #271/#368", {
+  dt <- lazy_dt(data.frame(x = 1, y = 2))
+  expect_snapshot_error(
+    select(dt, where(is.numeric))
+  )
+  expect_snapshot_error(
+    mutate(dt, across(!where(is.character), ~ .x + 1))
+  )
+})
+
 # dplyr verbs -------------------------------------------------------------
 
 test_that("n() is equivalent to .N", {
@@ -224,7 +319,7 @@ test_that("n() is equivalent to .N", {
   )
 })
 
-test_that("row_number() is equivalent .I", {
+test_that("row_number() is equivalent seq_len(.N)", {
   dt <- lazy_dt(data.frame(g = c(1, 1, 2), x = 1:3))
 
   expect_equal(
@@ -245,6 +340,21 @@ test_that("row_number(x) is equivalent to rank", {
   )
 })
 
+test_that("ranking functions are translated", {
+  df <- lazy_dt(tibble(x = c(1, 2, NA, 1, 0, NaN)))
+
+  res <- df %>%
+    mutate(percent_rank = percent_rank(x),
+           min_rank = min_rank(x),
+           dense_rank = dense_rank(x),
+           cume_dist = cume_dist(x))
+
+  expect_equal(pull(res, percent_rank), c(1 / 3, 1, NA, 1 / 3, 0, NA))
+  expect_equal(pull(res, min_rank), c(2L, 4L, NA, 2L, 1L, NA))
+  expect_equal(pull(res, dense_rank), c(2L, 3L, NA, 2L, 1L, NA))
+  expect_equal(pull(res, cume_dist), c(.75, 1, NA, .75, .25, NA))
+})
+
 test_that("scoped verbs produce nice output", {
   dt <- lazy_dt(data.table(x = 1:5), "DT")
 
@@ -282,6 +392,13 @@ test_that("`desc(col)` is translated to `-col` inside arrange", {
 
   expect_equal(show_query(step), expr(DT[order(-x)]))
   expect_equal(out$x, c("b", "a"))
+
+  # Can namespace `desc()`
+  step <- arrange(dt, dplyr::desc(x))
+  out <- collect(step)
+
+  expect_equal(show_query(step), expr(DT[order(-x)]))
+  expect_equal(out$x, c("b", "a"))
 })
 
 test_that("desc() checks the number of arguments", {
